/*****************************************************************************
	MML syntax coloring parser 
	Gordano Mail MetaLanguage - http://www.gordano.com/
	by Matt Lavallee (matt@mattl.com)
	Copyright (c) 2003 Matt Lavallee
/*****************************************************************************/

// TOKENS
#define DEFAULT					0
#define HTML					1
#define MML						2
#define MML_NUMBER				3
#define MML_COMMENT				4
#define MML_STRING				5
#define MML_TOKEN				6
#define MML_COMMAND				7
#define MML_FUNCTION			8
#define MML_OPERATOR			9
#define MML_SYMBOL				10
#define MML_CONSTANT			11
#define MML_DELIMITER			12

#define	HTML_START				40
#define HTML_STOP				41

#define MML_START				50
#define	MML_STOP				51

#define	MML_STRING_START		60
#define	MML_STRING_STOP			61

#define	MML_COMMENT_START		70
#define	MML_COMMENT_STOP		71

#define MML_TOKEN_START			80
#define MML_TOKEN_STOP			81


// STATES
#define IN_DEFAULT				0
#define IN_HTML					1
#define IN_MML					2
#define IN_STRING				3
#define IN_COMMENT				4
#define IN_TOKEN				5
#define ALL_STATES				[0..255]

// CharSets
#define ALPHA					'[A-Za-z]'
#define NUMERIC					'[0-9]'
#define NON_NUMERIC				'[^0-9]'
#define NON_ALPHA				'[^A-Za-z0-9]'
#define ALL_CHARS				'[A-Za-z0-9]'
#define NO_CHARS				'[]'
#define DONT_CARE				'[\x00-\xFE]'
#define	WHITE_SPACE				'[\x01-\s]'
#define	NOT_WHITE_SPACE			'[^\x01-\s]'

// COLORS
#define clLtSilver		15329769
#define clOrange		33023
#define clSoftBlue		12615680
#define clForest		6526464

// DEFINITION
%%language
Name                = 'Gordano, Ltd. Mail MetaLanguage'
Case                = __INSENSITIVE
Options             = __DEFAULT_OPTIONS
//StartState			= IN_MML
EditableStyles		('Default Text',		DEFAULT),
					('HTML',				HTML),
					('MML Text',			MML),
					('Numbers',				MML_NUMBER),
					('Comments',			MML_COMMENT),
					('Strings',				MML_STRING),
					('Parsed tokens',		MML_TOKEN),
					('Commands',			MML_COMMAND),
					('Functions',			MML_FUNCTION),
					('Operators',			MML_OPERATOR),
					('Symbols',				MML_SYMBOL),
					('Constants',			MML_CONSTANT),
					('Delimiters',			MML_DELIMITER)

ExampleText			= 	'<html>\n\
						\\t<title>MML Test</title>\n\
						\<#\n\
						\/*\n\
						\   MML Multiline \n\
						\   Comment */\n\
						\MyType = #domain#\\#account#\\type;\n\
						\if (BitIsSet(MyType, AccountTypeAlias)) {\n\
						\	testVar2 = "my mail\'s a success!";\n\
						\	testVar3 = 262192;\n\
						\}\n\
						\#></html>'

%%words
// Word/Seq.		Terminator				Token					Valid State
// HTML delimiters
'<'					DONT_CARE				HTML_START				[IN_DEFAULT]
'>'					DONT_CARE				HTML_STOP				[IN_HTML]

// MML delimiters
'<#'				DONT_CARE				MML_START				[IN_DEFAULT]
'#>'				DONT_CARE				MML_STOP				[IN_MML]

// String delimiters
'"'					DONT_CARE				MML_STRING_START		[IN_MML]
'"'					DONT_CARE				MML_STRING_STOP			[IN_STRING]

// Comment delimiters
'\/*'				DONT_CARE				MML_COMMENT_START		ALL_STATES
'*\/'				DONT_CARE				MML_COMMENT_STOP		[IN_COMMENT]

// Parsed Token delimiters
'#'					DONT_CARE				MML_TOKEN_START			[IN_MML]
'#'					DONT_CARE				MML_TOKEN_STOP			[IN_TOKEN]

// Separators
'('					DONT_CARE				MML_SYMBOL				[IN_MML]
')'					DONT_CARE				MML_SYMBOL				[IN_MML]
'{'					DONT_CARE				MML_SYMBOL				[IN_MML]
'}'					DONT_CARE				MML_SYMBOL				[IN_MML]
'['					DONT_CARE				MML_SYMBOL				[IN_MML]
']'					DONT_CARE				MML_SYMBOL				[IN_MML]
';'					DONT_CARE				MML_SYMBOL				[IN_MML]


%%handlers
// Token			Followers				Terminator				Use End?

%%kwtables
ForState [IN_MML] endswith NON_ALPHA (
	// Boolean operators
	MML_OPERATOR	is	'&&', '==', '||', '!=', '>=', '<='
	
	// Commands
	MML_COMMAND		is	'Def', 'Do', 'End', 'For', 'If', 'Else', 'Include', 'Return', 'Session', 'Global', 'While', 'Connect', 'Destroy'
	
	// Functions
	MML_FUNCTION	is	'addalias', 'adddomain', 'addrinrange', 'addsession', 'adduser', 'addtimed', 'addzip', 'asc', 'autoconnect',
						'autodisconnect', 'bitand', 'bitisset', 'bitmaskisset', 'bitreset', 'bitset', 'bound', 'chr',
						'closememberdb', 'closezip', 'collectfrompop', 'converttoaccount', 'converttodomain',
						'converttoemailaddress', 'converttohtml', 'converttojava', 'createsetup', 'date', 'datetimeformat',
						'decryptpassword', 'defaultlistparm', 'delalias', 'deldir', 'deldomain', 'deletememberrecord', 'delfile',
						'delsession', 'deltimed', 'deluser', 'dirsize', 'encryptpassword', 'enumrasentries', 'existdomain',
						'existfile', 'existuser', 'existvar', 'fileclose', 'fileeof', 'filereadline', 'fileopen', 'filewriteline',
						'filtermsg', 'findfiles', 'filereplace', 'folderappendmsg', 'folderclose', 'folderdelete', 'folderlist',
						'foldermsgsetstatus', 'foldermsgunsetstatus', 'foldermsgcheckstatus', 'folderopen', 'foldergetmessagecount',
						'getallmembers', 'gethostedips', 'gethostname', 'getipaddress', 'getlocaldomain', 'getlocalips',
						'getmemberrecord', 'getmxrecord', 'getos', 'getstatus', 'getusersoftype', 'importmembers', 'instr',
						'isdialupenabled', 'isconnected', 'isdate', 'isdomain', 'isinteger', 'isipaddress', 'ismemberoflist',
						'isvalidstr', 'isvalidpassword', 'iswildcard', 'keyexpired', 'keyexpireson', 'keyabouttoexpire',
						'keyactivate', 'keyint', 'keystring', 'keyupgrade', 'keyuserid', 'left', 'len', 'listversion',
						'lsappend', 'lsdelete', 'lsdeleteelement', 'lselement', 'lsfind', 'lsfirstmatch', 'lslength',
						'lsmatch', 'lsreplace', 'lsorder', 'lssubset', 'location', 'log', 'match', 'memberformat',
						'mid', 'msgcreate', 'msgaddheader', 'msgaddbody', 'msgaddfile', 'msgaddrecipient', 'msgaddattachment',
						'msgclose', 'msgcopy', 'msgdecode', 'msgendoflines', 'msgreadfirstline', 'msgreadnextline',
						'msgremoveheader', 'odbcinstalled', 'openmemberdb', 'openzip', 'print', 'proxyagecache',
						'readnextmemberrecord', 'resolve', 'right', 'runexecutable', 'sendnotification', 'serverdsnexists', 
						'servicestart', 'servicestatus', 'servicestop', 'setmemberrecord', 'setpassword', 'shutdownsystem',
						'sleep', 'hsprintf', 'sqlquery', 'sqlget', 'toint', 'trim', 'urlencode', 'urldecode', 'verifyuser',
						'wildcardmatch', 'wildcardfiltermsg', 'lselement2', 'lsappend2', 'lslength2', 'diffdate', 'time',
						'getlogonuser', 'deluserfiles', 'deldomainfiles', 'filterusersoftype', 'checkserver', 'loggedonusers',
						'getselectorjavaappletparameter', 'isabsolutefilename', 'getmailboxname', 'getremoteconnectionaddr',
						'getlocalserveraddr', 'getsessionvariables', 'getconnectionvariables', 'converttotime', 'createdirectory',
						'isvalidemailaddress', 'filterdomainsoftype', 'searchfile', 'weakencryptvalue', 'weakdecryptvalue',
						'getloadsharingserver', 'getloadsharingserverlist', 'isvaliddate', 'checkpassword',
						'purgednscache', 'killscript', 'listrunningscripts', 'sethttpresponsestatus', 'addhttpresponsehead',
						'getuid'
	
	// Constants
	MML_CONSTANT	is 	'ACCESSRIGHTDOMAIN', 'ACCESSRIGHTGUEST', 'ACCESSRIGHTJUCE', 'ACCESSRIGHTLOG',
						'ACCESSRIGHTSUPPORT', 'ACCESSRIGHTSYSTEM', 'AccountTypeAlias', 'AccountTypeAutoResponder',
						'AccountTypeDLL', 'AccountTypeForward', 'AccountTypeList', 'AccountTypeListAll',
						'AccountTypeListManager', 'AccountTypeMailbox', 'AccountTypeMoved', 'AccountTypeNTDatabase',
						'AccountTypeNTMail', 'AccountTypeRobot', 'AccountTypeScript', 'AccountTypeSuperList',
						'AccountTypeTemplate', 'AccountTypeTranslate', 'AccountTypeVirtualPOP', 'ALL_SERVICE',
						'DomainTypeDLL', 'DomainTypeFull', 'DomainTypePOP', 'DomainTypeRobot', 'DomainTypeVirtual',
						'FALSE', 'FILE_APPEND', 'FILE_BINARY', 'FILE_READ', 'FILE_TEXT', 'FILE_WRITE',
						'FOLDER_MSG_ANSWERED', 'FOLDER_MSG_DELETED', 'FOLDER_MSG_DIGEST', 'FOLDER_MSG_DRAFT',
						'FOLDER_MSG_FLAGGED', 'FOLDER_MSG_MODERATED', 'FOLDER_MSG_POSTED', 'FOLDER_MSG_READ',
						'FOLDER_MSG_RECENT', 'FOLDER_MSG_SEEN', 'FOLDER_MSG_WARNING', 'JOIN_LISCOMMAND',
						'KEY_CUSTOMER', 'KEY_ISDEMO', 'KEY_PRODUCTACTIVE', 'KEY_PRODUCTNAME', 'KEY_USERS',
						'KEY_USERPERM_LIST', 'LEAVE_LISTCOMMAND', 'LEAVE_REASON_BAD_MESSAGE_LIMIT', 'LEAVE_REASON_BANNED',
						'LEAVE_REASON_EXPIRED', 'LEAVE_REASON_MEMBER', 'LINUX', 'LISTACCESSALLOWED',
						'LISTACCESSMODERATE', 'LISTACCESSPASSWORD', 'LISTACCESSSELFONLY', 'LISTPARM_ACCESSANYONE',
						'LISTPARM_ACCESSFILE_ANYONE', 'LISTPARM_ACCESFILE_ONLY', 'LISTPARM_ACCESSMEMBER_ANYONE',
						'LISTPARM_ACCESSMEMBER_ONLY', 'LISTPARM_ACCESSMEMBER_PASSWORD_ANYONE',
						'LISTPARM_ACCESSMEMBER_PASSWORD_ONLY', 'LISTPARM_ACCESSMODERATOR', 'LISTPARM_ACCESSNO_ACCESS',
						'LISTPARM_ACCESSONLY', 'LISTPARM_ACCESSPASSWORD_ANYONE', 'LISTPARM_ACCESSPASSWORD_ONLY',
						'LISTPARM_ACCESSWILDCARD', 'LISTPARM_ACTIVE', 'LISTPARM_DO_NOTHING', 'LISTPARM_INDEX_DIGESTSTR',
						'LISTPARM_MIME_DIGESSTR', 'LISTPARM_ODBC_RO_TYPE', 'LISTPARM_ODBC_TYPE', 'LISTPARM_PAUSED',
						'LISTPARM_STOPPED', 'LISTPARM_SUSPEND_IGNORE', 'LISTPARM_SUSPEND_TO_ADDRESS',
						'LISTPARM_SUSPEND_TO_MODERATOR', 'LISTPARM_SUSPEND_TO_OWNER', 'LISTPARM_TEXDIGESTSTR',
						'LISTPARM_TO_ADDRESS', 'LISTPARM_TO_MODERATOR', 'LISTPARM_TO_OWNER', 'LISTPARM_TO_SENDER',
						'LISTPARM_V4_FILE_TYPE', 'LOG_DNS', 'LOG_FAILURE', 'LOG_JUCE', 'LOG_PROGRESS', 'LOG_PROTOCOL',
						'LOG_RETURN', 'LOG_SPAM', 'LOG_STATS', 'MEMBER_TYPE_BANNED', 'MEMBER_TYPE_BEING_MODERATED',
						'MEMBER_TYPE_CONFIRM', 'MEMBER_TYPE_DELETE', 'MEMBER_TYPE_JOIN_EXPIRED', 'MEMBER_TYPE_MEMBER',
						'MEMBER_TYPE_OLD', 'MEMBER_TYPE_ZOMBIE', 'MSG_DROP', 'MSG_QUEUE', 'MSG_SEND', 'NTMAIL_INSTALL_DIR',
						'PasswordPolicyDigit', 'PasswordPolicyLetter', 'PasswordPolicySymbol', 'POSTSERVICE', 'PRODUCTJUCE',
						'PRODUCTNTLIST', 'PRODUCTNTMAIL', 'PRODUCTVIRUSSCANNER', 'PROXY_PURGE_CREATED',
						'PROXY_PURGE_LASTUSED', 'RESUME_LISCOMMAND', 'RFC822_ADD_FROM', 'RFC822_ADD_TO',
						'RFC822_ADD_TO_ALWAYS', 'RFC822_STRICT', 'SMTP_SERVICE', 'SOLARIS', 'SUSPEND_LISTCOMMAND',
						'TRUE', 'WIN95', 'WINNT', 'WWW_SERVICE'
)

%%tokens
// 	Token			Start					Followers				Terminator				Use End?
InState IN_DEFAULT (
	DEFAULT			DONT_CARE				DONT_CARE?				DONT_CARE				_discard_
)

InState IN_HTML (
	HTML			DONT_CARE				DONT_CARE?				DONT_CARE				_discard_
)

InState IN_MML (
	DEFAULT				'[^A-Za-z0-9_$]'	DONT_CARE?				DONT_CARE				_discard_
	MML					'[A-Za-z_$]'		'[A-Za-z0-9_$]'?		'[^A-Za-z0-9_$]'		_discard_
	MML_NUMBER			NUMERIC				NUMERIC?				NON_NUMERIC				_discard_
)

InState IN_STRING (
	MML_STRING		'[^"\\]'				DONT_CARE?				DONT_CARE				_discard_
	MML_STRING		'\\'					DONT_CARE				DONT_CARE				_discard_
)

InState IN_COMMENT (
	MML_COMMENT		DONT_CARE				DONT_CARE?				DONT_CARE				_discard_
)

InState IN_TOKEN (
	MML_TOKEN		DONT_CARE				DONT_CARE?				DONT_CARE				_discard_
)

%%states
// Token			(-[State to Disable]		+[State to Enable])
HTML_START			(-ALL_STATES				+[IN_HTML])
HTML_STOP			(-[IN_HTML]					+[IN_DEFAULT])

MML_START			(-ALL_STATES				+[IN_MML])
MML_STOP			(-[IN_MML]					+[IN_DEFAULT])

MML_STRING_START	(-[IN_MML]					+[IN_STRING])
MML_STRING_STOP		(-[IN_STRING]				+[IN_MML])

MML_COMMENT_START	(-ALL_STATES				+[IN_COMMENT])
MML_COMMENT_STOP	(-[IN_COMMENT]				+[IN_MML])

MML_TOKEN_START		(-[IN_MML]					+[IN_TOKEN])
MML_TOKEN_STOP		(-[IN_TOKEN]				+[IN_MML])

%%map
// Token			Effect Definition
DEFAULT				DEFAULT
HTML				HTML
HTML_START			HTML
HTML_STOP			HTML
MML					MML
MML_NUMBER			MML_NUMBER
MML_COMMENT			MML_COMMENT
MML_COMMENT_START	MML_COMMENT
MML_COMMENT_STOP	MML_COMMENT
MML_STRING			MML_STRING
MML_STRING_START	MML_STRING
MML_STRING_STOP		MML_STRING
MML_TOKEN			MML_TOKEN
MML_TOKEN_START		MML_TOKEN
MML_TOKEN_STOP		MML_TOKEN
MML_COMMAND			MML_COMMAND
MML_FUNCTION		MML_FUNCTION
MML_OPERATOR		MML_OPERATOR
MML_SYMBOL			MML_SYMBOL
MML_CONSTANT		MML_CONSTANT
MML_DELIMITER		MML_DELIMITER
MML_START			MML_DELIMITER
MML_STOP			MML_DELIMITER


%%effects
// Token			TextStyle				Foreground				Background
DEFAULT				[]						clBlack					clWhite*
HTML				[]						clBlack					clLtSilver
MML					[]						clBlack*				clWhite*
MML_NUMBER			[]						clGreen					clWhite*
MML_COMMENT			[fsBold]				clOrange				clWhite*
MML_STRING			[]						clTeal					clWhite*
MML_TOKEN			[]						clBlue					clWhite*
MML_COMMAND			[fsBold]				clNavy					clWhite*
MML_FUNCTION		[fsBold]				clBlack*				clWhite*
MML_OPERATOR		[]						clBlue					clWhite*
MML_SYMBOL			[]						clBlack*				clWhite*
MML_CONSTANT		[fsBold]				clNavy					clWhite*
MML_DELIMITER		[fsBold]				clSoftBlue				clWhite*

